module class_default_select_type_user1
use class_default_select_type_user
implicit none

end module class_default_select_type_user1

program select_type4
implicit none

print *, "running select_type4 main program"

end program select_type4
