#ifndef BINDC_02C
#define BINDC_02C

void driver();
void print_ptr1(int n, float *A);
void print_ptr2(int *n, float *A);

void callback1(int n, float *A);
void callback1b(int n, float *A);
void callback2(int *n, float *A);
void callback2b(int *n, float *A);

#endif // BINDC_02C
