program logical4
! this program checks logical operators
implicit none
    
! variable declaration
logical :: a, b, c
       
! assigning values
a = 2
b = -1
c = 0

print *, a, b, c
       
end program logical4
