function a()
integer :: a
end function

function b()
real :: b
end function

function c()
logical :: c
end function

function d() result(r)
integer :: r
end function

function e() result(r)
real :: r
end function

function f() result(r)
logical :: r
end function
