      program format_03
 1000 FORMAT
     +  (/' --- PROBLEM SIZE:'/
     +    '            N = ',I5,
     +    '          (NUMBER WITH NONZERO WEIGHT = ',I5,')'/
     +    '           NQ = ',I5/
     +    '            M = ',I5/
     +    '           NP = ',I5,
     +    '          (NUMBER UNFIXED = ',I5,')')
      end program
