      program main
          integer k, l, m, n
          do 100 k=1,2
            do 100 l=1,3
              do 100 m=1,4
                do 100 n=1,5
                  print *, k, l
 100      continue
      end program

