program reserved_02
implicit none
if (integer() /= 1) error stop

contains

integer function integer()
integer function
function = 5
function = function/function
integer = function
end function

end program
