submodule(sorting_separate_compilation_24) radix_sort_separate_compilation_24
    implicit none
contains

    module subroutine sort(n)
        integer, intent(inout) :: n
        n = 5
    end subroutine

end submodule radix_sort_separate_compilation_24