program reserved_01
implicit none
if (end(1, 2) /= 2) error stop
if (end(2, 1) /= 2) error stop

contains

integer function end(return,&
integer)result(function);;;;&
integer integer,return;;;;;;&
;;function=return;if(return>&
integer)return;;;;;function=&
integer;;end function end;;;;

end program
