program array2
implicit none

real, dimension(5) :: a, b
integer, dimension(3) :: c
logical, dimension(2) :: d

real, dimension(2,3) :: e
integer, dimension(3,4) :: f
logical, dimension(5,2) :: g

real, dimension(2,3,4) :: h
integer, dimension(3,4,3) :: i
logical, dimension(5,2,2) :: j

end program
