subroutine dqc25c(f)
   double precision f
   call dqk15w(f)
   print *, f(hlgth)
end

function func(f)
   double precision c
   call sub2(c)
   print *, c(d)
end function

program main
   double precision a
   call sub(a)
   print *, a(b)
end program
