program write_01
implicit none

write(*,*) "a", 2
write(*,*)
write(*) a
write(out_str,'(" Found index ", i9," but item is not in list.")') index
write(out_string,*) 'Scalar_in = ', scalar_in
write(out_string, 10) int_src, int_given, int_expected
write(string,fmt='(f6.2)') max_angle
write(fmt,'("(i",i0,".",i0,")")') n, n
write(unit=*, fmt='(a)') trim(prog) // ': ' // message
write(unit=0,fmt='(a,i4.4)') "Assertion failed at " // file // ":", line
write(*,fmt='(a,l1)') 'exactly_conservative= ',exactly_conservative
write(unit=lun) vec
write(*,'(a,a)',advance='no') trim(input_string),": "
WRITE( ounit, FMT=303, ADVANCE='NO' )
WRITE( *, 201)     ( star, i = 1, 80 )
WRITE( *, '(1X,A)') 'Success!'
WRITE( ounit, * )

print *, "2"
print *
print "(a, es22.14)", "Ekin:     ", Ekin

end program
