module fpm_cmd_install_modules_34
use fpm_modules_34, only : build_model
implicit none

end module fpm_cmd_install_modules_34

program modules_34
implicit none

print *, "running modules_34 program"

end program
