      integerfunctionfunctionf(n)
      integerfunctionfunctionf(n)
      integern
      endfunctionfunctionf

      integerfunctionf(endfunctionf)
      integerendfunctionf
      endfunctionf=4
      endfunctionf

      integerfunctiong(integerx)
      integerintegerx
      integerx
      integerx=5
      x=5
      integerx=5
      endfunctiong

      integerfunctiong2(integerx)
      integerintegerx
      integer::x=5
      x=5
      integerx=5
      endfunctiong2
