      program main
      integer k
      do k=1,10
        print *, k
      end do



      do 80 k=1,10
        print *, k
   80 continue
      do 800 k=1,10
        print *, k
  800 continue
      do 8000 k=1,10
        print *, k
 8000 continue


      end program
