program main
    implicit none
    integer :: x(3), a
    real :: z(2)
    x(1) = 1
    x(2) = 2
    x(3) = 3
    a = 10249275
    z(1) = 20.420
    z(2) = 3204.02

    print *, "x is printed:", x, 42, 53, z, a

 end program
