      program assign3
   10 assign 30 to next
   20 GO TO next,(30, 50, 70, 110)
      assign 50 to next
      assign 70 to next
      assign 110 to next
   30 print *, 30
   50 print *, 50
   70 print *, 70
  110 print *, 110
      end program

