      program main
      integer k,l,m,n
      do k=1,10
         print *, k
       enddo



       do 80 l=1,10
         print *, l
   80  enddo
       do 800 m=1,10
         print *, m
  800  enddo
       do 8000 n=1,10
         print *, n
 8000  enddo


         end program
