        subroutine f()
            integer :: a, b, c, d, e, g, h, i, j, k
            common / block_1 / a, b, h / block_2 / c, d, / block_3 / e,
     &           g
            common / block_4 / i, / block_5 / j, k
        end subroutine
