program reserved_03
implicit none
integer endsubroutine
call integer
if (endsubroutine /= 1) error stop

contains

subroutine integer
integer subroutine
subroutine = 1
endsubroutine &
= subroutine/subroutine
endsubroutine &


; end program
