SUBROUTINE ITAIRY()
IMPLICIT DOUBLE PRECISION (A-H,O-Z)
DIMENSION A(2)

DATA A/.569444444444444D0,.891300154320988D0/
print *, A(1), A(2)
IF (A(1) /= .569444444444444D0) ERROR STOP
IF (A(2) /= .891300154320988D0) ERROR STOP
RETURN
END

PROGRAM MAIN
    CALL ITAIRY()
END PROGRAM 
