
SUBROUTINE SSYR2(UPLO)
    CHARACTER UPLO
    INTEGER INFO
    LOGICAL LSAME
    EXTERNAL LSAME
    INFO = 0
    IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
        INFO = 1
    ELSE 
        INFO = 2
    END IF

    RETURN
END
  