module tomlf_build
   use tomlf_build_array, only : get_value
   use tomlf_build_keyval, only : get_value
   implicit none


end module tomlf_build

program modules_27
implicit none

print *, "running modules_27 program"

end program
