module sorting_separate_compilation_24
    implicit none

    interface
        module subroutine sort(n)
            integer, intent(inout) :: n
        end subroutine
    end interface
    
end module sorting_separate_compilation_24