! This is a test program that CMake uses to check that the Fortran compiler can
! compile Fortran 90 code.
      PROGRAM TESTFortran90
      integer stop ; stop = 1 ; do while ( stop .eq. 0 ) ; end do
      END PROGRAM TESTFortran90
