module module_document_symbols1
    implicit none
    integer :: xy, yz
contains
subroutine sub()
    implicit none
end subroutine
subroutine sub2()
    implicit none
end subroutine
end module module_document_symbols1
