recursive pure integer function a()
end function

pure integer elemental function b()
end function

elemental pure integer function c()
end function

elemental pure function cb()
integer cb
end function

integer pure recursive function d()
end function
