SUBROUTINE COBYLA (CALCFC)
IMPLICIT DOUBLE PRECISION (A-H,O-Z)
EXTERNAL CALCFC
CALL COBYLB (CALCFC)
RETURN
END

SUBROUTINE COBYLB (CALCFC)
IMPLICIT DOUBLE PRECISION (A-H,O-Z)
EXTERNAL CALCFC
CALL CALCFC(CON)
RETURN
END
