integer function a()
implicit none
integer :: b, c
real :: d, e, f
integer :: d1, e1, f1
real*8 :: g, h, i
complex :: j, k
complex*8 :: l, m
complex *16 :: n
double precision :: o, p
integer :: Y(e1)
real :: X(d1, e1, f1)
Y(1) = 3
X(1, 1, 1) = 3
a = 1
b = 2
d = 3
f = 4
h = 5
i = 6
l = 7
m = 8
o = 9
p = 10
end function
