integer function a(d, e)
implicit integer (a,b-c), integer*4 (d-e), integer*8 (f-g), real (h)
implicit real*4 (i-k), real*8 (l), complex (m, n), complex*8 (o)
implicit complex*16 (p), double precision (q), double precision (r)
integer :: Y(e)
real :: X(d, e, e)
real :: b
Y(1) = 3
X(1, 1, 1) = 3
a = 1
b = 2
d = 3
f = 4
h = 5
i = 6
l = 7
m = 8
o = 9
p = 10
q = 11
r = 12
end function
