real(kind=c_float) r
real(c_float) r
character(kind=c_char) c
character(*) c
character(len=*) c
character(len=*, kind=c_char) c
character(*, kind=c_char) c
character(kind=c_char, len=*) c
character(:) c
character(len=:) c
character(len=:, kind=c_char) c
character(:, kind=c_char) c
character(kind=c_char, len=:) c
character(n) c
character(len=n) c
character(len=n, kind=c_char) c
character(n, kind=c_char) c
character(kind=c_char, len=n) c
