      subroutine f(a)
          integer a
          print *, a
      end subroutine
c this is a test comment
      program main
        integer , dimension (3,3) :: z
        integer a, b
        dimension a(2,2), b(10,10)
        integer, dimension(2,2) :: d
        call f(1)
      end

