program test_hybrd

    implicit none

    contains

    subroutine compare_solutions(x)

    real, dimension(:), intent(in) :: x 
    real, dimension(:), allocatable :: temp
    end subroutine compare_solutions
end program
