submodule(julienne_formats_m_submodule_18b) julienne_formats_s_submodule_18b
  implicit none

contains

  module procedure separated_values
    format_string = "(*(G25.20,:,','))"
  end procedure

end submodule julienne_formats_s_submodule_18b
