integer function a()
end function

real function b()
end function

logical function c()
end function

integer function d() result(r)
end function

real function e() result(r)
end function

logical function f() result(r)
end function
