program logical1
! this program checks logical operators
implicit none
    
! variable declaration
logical :: a, b
       
! assigning values
a = .true.
b = .false.

print *, a, b
       
end program logical1
