module string_03_mod
implicit none
character(len = 5), parameter :: intro = "I've "
end module

program string_03
use string_03_mod, only: intro
implicit none

character(len = 8) :: verb
character(len = 5) :: posit
character(len = 4) :: title
character(len = 7) :: last_name
character(len = 29) :: combined

verb = "learned "
posit = "from "
title = 'the '
last_name = 'best'

combined = intro//verb//posit//title//last_name//"."

print *, combined

end program
